/**
 * @file fsd_defines.h
 * @brief Implements debuging macros for deamon part of filesystem
 * @author Michal Podzimek, Michal Ficek, Tomas Pop, MFF UK, 2007
 *
 * @addtogroup q_mysqlfs_deamon MySQL FS Deamon
 * @{
 * @addtogroup g_mysqlfs_deamon_device Device
 * @{
 * 
 */


#ifndef FSD_DEFINES_H_
#define FSD_DEFINES_H_


/** definition of version of communication protocol */
#define FSD_VERSION "MYSQL1.1"


/** @name constants for symbols for commands
 * 
 * @{
 */
#define TEST			"TS"
#define CREATE 			"CF"
#define MKDIR 			"MD"
#define CLEAR_FILE 		"CR"
#define ADD_BLOCK 		"AB"
#define REMOVE_FILE 	"RF"
#define REMOVE_DIR 		"RD"
#define GET_BLOCK		"GB"
#define LOOKUP			"LU"
#define READ_DIR		"RE"
#define FILE_INFO		"FI"
#define DIR_INFO		"DI"
#define READ_INODE		"RI"
#define CMD_OK			"OK"
#define CMD_ER			"ER"
#define REPLACE_BLOCK 	"RB"
#define MOVE_FILE 		"MV"
#define RENAME_FILE 	"DN"
#define RENAME_DIR 		"DD"
#define TRUNCATE_FILE 	"TR"
#define APPEND_FILE 	"AP"
/** @} */


/** @name constants for numbers of commands
 * 
 * @{
 */
#define UNKNOWN_COMMAND_NR 		1000
#define TEST_NR					1001
#define CREATE_NR 				1002
#define MKDIR_NR 				1003
#define CLEAR_FILE_NR 			1004
#define ADD_BLOCK_NR 			1005
#define REMOVE_FILE_NR 			1006
#define REMOVE_DIR_NR 			1007
#define GET_BLOCK_NR			1008
#define LOOKUP_NR				1009
#define READ_DIR_NR				1010
#define CMD_OK_NR				1011
#define CMD_ER_NR				1012
#define DIR_INFO_NR				1013
#define FILE_INFO_NR			1014
#define READ_INODE_NR			1015
#define REPLACE_BLOCK_NR 		1016
#define MOVE_FILE_NR 			1017
#define RENAME_FILE_NR 			1018
#define TRUNCATE_FILE_NR 		1019
#define RENAME_DIR_NR 			1020
/** @} */


/** @name constants for restrictions of length
 * 
 * @{
 */
#define MAX_NAME_LENGTH 	64
#define MAX_ID_LENGTH 		11
#define MAX_SIZE_LENGTH 	11
#define MAX_VERSION_LENGTH 	10
#define MAX_COMMAND_LENGTH 	3
#define MAX_PARAM_LENGTH 	64
#define MAX_BLOCK_LENGTH  	4096

/* max message length = "version command param data\n" */
#define MAX_MESSAGE_LENGTH 	5192
#define MAX_QUERY_LENGTH 	300
/** @} */


/** @name constants for connection to char device
 * 
 * @{
 */
#define CHAR_DEVICE_NAME 	"fsc_dev"
#define FSC_DEVICE_LOCATION "/dev/fsc_dev"
/** @} */





/** @name constants for functions return values
 * 
 * @{
 */
#define DEFINITION_BASE 			100

#define IS_FILE 					(+ (DEFINITION_BASE + 6))
#define IS_DIR 						(+ (DEFINITION_BASE + 7))
#define OK 							(+ (DEFINITION_BASE + 8))
	
#define ENAME_NOT_EXISTS	 		(- (DEFINITION_BASE + 11))
#define EINODE_NOT_EXISTS	 		(- (DEFINITION_BASE + 11))
#define EDB_ERROR 		 			(- (DEFINITION_BASE + 1)) 
#define ENAME_EXISTS 		 		(- (DEFINITION_BASE + 3))
#define EDIR_NOT_EXISTS 	 		(- (DEFINITION_BASE + 2))
#define EDIR_IS_NOT_EMPTY 			(- (DEFINITION_BASE + 4))
#define EBLOCK_NOT_EXISTS 	 		(- (DEFINITION_BASE + 5))
#define EFILE_NOT_EXISTS	 		(- (DEFINITION_BASE + 9))
#define EDB_CONNECT_ERROR       	(- (DEFINITION_BASE + 10))
#define EFSC_DEVICE_OPEN_ERROR		(- (DEFINITION_BASE + 12))
#define EFSC_DEVICE_NOT_OPEN    	(- (DEFINITION_BASE + 13))
#define EGENERAL_ERROR 		 		(- (DEFINITION_BASE + 14))
#define EPROTOKOL_VERSION_ERROR 	(- (DEFINITION_BASE + 15))
#define EDIR_NAME_NOT_EXISTS 		(- (DEFINITION_BASE + 16))
#define EFILE_NAME_NOT_EXISTS 		(- (DEFINITION_BASE + 17))
#define EPARENT_NOT_EXISTS 			(- (DEFINITION_BASE + 18))
/** @} */


/**
 * When a directory list is received, is nice to parse data part of a command to 
 * the list of directory item. 
 */
struct read_dir_item {
	/** type of item, "f" for file or "d" for dir  */
	char type[2];
	/** id of item in db, its same as id of inode  */
	char id[MAX_ID_LENGTH + 1];
	/** name of item  */
	char name[MAX_NAME_LENGTH + 1];
	/** pointer to next item or null if its last item  */
	struct read_dir_item *next;
};


/**
 * @see read_dir_item
 */
typedef struct read_dir_item rd_item;


#endif

/**
 *
 * @}
 * 
 * @}
 * 
 */
