/**
 * @file fsd_mysql.h
 * @brief Header file for implementation of mysql commands to db, file fsd_mysql.c
 * @author Michal Podzimek, Michal Ficek, Tomas Pop, MFF UK, 2007
 *
 * @addtogroup q_mysqlfs_deamon MySQL FS Deamon
 * @{
 * @addtogroup g_mysqlfs_deamon_db Database Operations
 * @{
 * 
 * This file contents declaration of functions which established and close connection to 
 * database. And function which execute commands that interacts with database.
 * 
 */


#ifndef _FSD_MYSQL_H_
#define _FSD_MYSQL_H_


#include <mysql/mysql.h>
#include "fsd_defines.h"


/** @name declaration of variables which supports connection to database
 * 
 * @{
 */
char *server = 		"localhost";
char *user = 		"mysqlfs";
char *password = 	"mysqlfs";
char *database = 	"mysqlfs";
/** @} */


/** @name declaration of functions for connect and close database
 * 
 * @{
 */
MYSQL *fsd_db_connect(char *server, char *user, char* password, char *database);
void fsd_db_close(MYSQL *conn);
/** @} */


/** @name declaration of functions for filesystem to use database
 * 
 * @{
 */
int fsd_test(MYSQL *conn);
int fsd_create(MYSQL *conn, char *dir, char *filename, char *return_text);
int fsd_mkdir(MYSQL *conn, char *dir, char *dirname, char *return_text);
int fsd_clear_file(MYSQL *conn, char *file, char *return_text);
int fsd_add_block(MYSQL *conn, char *file, int block_size, char *block, char *return_text);
int fsd_replace_block(MYSQL *conn, char *file, char *offset, int block_size, char *block, char *return_text);
int fsd_remove_file(MYSQL *conn, char *file, char *return_text);
int fsd_remove_dir(MYSQL *conn, char *dir, char *return_text);
int fsd_get_block(MYSQL *conn, char *file, char *offset, int *block_size, char *block, char *return_text);
int fsd_lookup(MYSQL *conn, char *dir, char *name, char *return_text);
int fsd_read_inode(MYSQL *conn, char *inode_nr, char *return_text);
int fsd_read_dir(MYSQL *conn, char *dir, rd_item **rd_list, char *return_text);
int fsd_file_info(MYSQL *conn, char *file, char *return_text);
int fsd_dir_info(MYSQL *conn, char *dir, char *return_text);
int fsd_move_file(MYSQL *conn, char *file, char *dir, char *return_text);
int fsd_rename_file(MYSQL *conn, char *file, char *name, char *return_text);
int fsd_rename_dir(MYSQL *conn, char *dir, char *name, char *return_text);
int fsd_truncate_file(MYSQL *conn, char *file, char *offset, char *return_text);
/** @} */


#endif


/**
 *
 * @}
 * 
 * @}
 * 
 */
