/**
 *
 * \mainpage [MYSQLFS - Linux  VFS driver using userspace deamon - Programmers Documentation]
 *
 * This is implementation of linux filesystem driver  storing data to a mysql database.
 * it uses userspace deamon to comunicate with DB. It was written as semestral work for subject
 * "Operacni systemy" (Operating systems) on <a href="http://mff.cuni.cz">MFF UK</a>, Czech Republic 
 * by team <b>Michal Ficek, Michal Podzimek, Tomas Pop</b> in year 2007. 
 *  
 * Principle you can see on a picture bellow
 * 
 *
 * <pre>
 *  _________________________________________________________________    _______
 * |                                                                 |  |       |
 * |                      LOCAL COMPUTER                             |  |REMOTE |
 * | _______________________________________    __________________   |  |MYSQL  |
 * ||  KERNEL                               |  | USERSPACE        |  |  |SERVER |
 * || -----------------     --------------  |  | ---------------- |  |  |       |
 * || | VFS DRIVER    | <-> |CHAR DEVICE |<-|->||USERSPACE DEAMON||<-|->|       |
 * || -----------------     --------------  |  |----------------- |  |  |       |
 * ||_______________________________________|  |__________________|  |  |       |
 * |_________________________________________________________________|  |_______|
 * 
 * </pre>
 * 
 * <b>Most inportant files:</b>
 *	@see kernel_module/fsc_device.c 		.. Char device implementation
 *	@see kernel_module/fs_inode.c 			.. Inode operations 
 *	@see kernel_module/fs_dir.c 			.. Dir operations 
 *	@see kernel_module/fs_file.c 			.. File operations
 *	@see kernel_module/fs_main.c 			.. Module "main" file (includes module_init() call)  
 *	@see deamon/fsd_main.c 				.. Deamon main file
 *	@see deamon/fsd_mysql.c 			.. Comand to SQL Query implementation
 *   
 * 
 * <h3>Software requirements</h3>
 * Filesystem was developed and tested on suse linux 10.2.
 *  
 * <h4>Kernel version</h4>
 * Supported kernel version is 2.6.18. Small changes are needed for kernel 2.6.16 or 2.6.20 
 * (With this kernel is not possible to build the filesystem)
 *
 * <h4>Libraries and sources needed</h4>
 * To buid this you need
 * <ul>
 * 	<li>Standart libraries</li>
 * 	<li>Kernel sources and headers</li>
 *  <li>Mysql-devel C library</li> 
 * </ul>
 * 
 * <h4>Tools</h4>
 *
 * You need a standart tool-chain, make, gcc ..., for running you need a MySQL databaze running
 * <h3>How to install and runn</h3>
 * <h4>Dataze</h4>
 * <ul>
 *  <li>Create a DB "mysqlfs"</li>
 *  <li>Execute SQL script from /doc/db/install.sql</li>	
 * </ul>
 * 
 * <h4>Filesystem module and deamon</h4>
 * run as root script startfs.sh 
 * <h4>Tests</h4>
 * run as root script filesystemtest.sh 
 *      
 * <h3>Some Implementation Notes</h3>
 * 
 * <h4>PHP frontend</h4>
 * Is quite buggy, but is a way, how to visualize what is going on in th DB. It's not official part of project.
 *  
 * <h4>Other documentation</h4>
 * 
 * Some other documentation is in directory doc/
 * Interesting may be espetialy doc/protokol/protokol.txt (protokol  between userspace and deamon specification)
 * 
 * All parts are desctibed on the begining of corresponfing .c file
 * 
 * <h4>Char device implementation</h4>
 * Chardevice is implemented strangely. In some case cases it doesn't behave as a char device
 * but as a block device. See all notes in kernel_module/fsc_device.c if you want to use it 
 * for something else than it it was done for.
 * 
 * 
 * <h3>Functionality of the filesystem SOME WARNINGS</h3>
 * This is just a seminar work, when you want to MySQL filesystem, it's done much more
 * correctly using fuse. So we didn't waste a time with things like users, hardlinks, softling...
 * 
 * Filesystem offer basic functionalyty, it mean read, write, create/unlink files,
 * create/remove directory. 
 * Other operations are not supported and behavior is <b><span style="color:red;">UNDEFINED</span></b>
 * (For example chmod seems to work, using generic one, but nothing is written to database, so after unmount
 * all chmod changes are lost.
 * )
 * 
 * <h3>Known bugs</h3>
 * 
 * We have tested filesystem, but we know abou several bugs. It would be possible to find them but,
 * other groups has moustly read only filesystems. There is no more time to debudd all these problems
 *  
 * <ul>
 * <li>Maximum file name length supported is 64 chars. This is not tested in filesystem and can cause
 * a serious problems. Its just few minutes to corect this</li>   
 * <li>May be some other :-)</li>  
 * </ul>
 * 
 * <h3>Opened questions</h3>
 * 
 * There are some details we didnt bring to end, because we know that this fs will never be used in
 * real life. 
 *  
 * <ul>
 * <li>Databaze connection information are hard coded in deamon source code</li>   
 * <li>May be some other :-)</li>  
 * </ul> 
 */



/**
 * @file fs_main.c
 * @brief Defines functions for kernel part of filesystem to manipulate with directories
 * @author Michal Podzimek, Michal Ficek, Tomas Pop, MFF UK, 2007
 *
 * @addtogroup q_mysqlfs_kernel MySQL FS Kernel Module
 * @{
 * @addtogroup g_mysqlfs_kernel_lib_c Library for RKFS
 * @{
 * 
 */


#ifndef __KERNEL__
#  define __KERNEL__
#endif


#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <linux/statfs.h>
#include <linux/mm.h>
#include <asm/uaccess.h>
#include <asm/errno.h>
#include <linux/buffer_head.h>
#include <linux/pagemap.h> /* unlock_page */
#include <linux/proc_fs.h>
#include <asm/atomic.h>
#include <linux/string.h>


#include "fsc_device.h"
#include "rkfs.h"
#include "parser.h"


/** variable for storing result form function query */
int result;

/** structure for storing parsed command */
struct_command ret_command; /* BIG ISSUE NR.1 - ask Mr.Child */

/** buffer for output messages */
char out_buf[OUT_BUF_LENGTH];


/**
 * @brief Function to send message to deamon 
 * 
 * Send message to deamon, accept answer message and parse it.
 * 
 * @param command command to be executed by deamon
 * @param command_length length of command without trailing zero
 * @return RKFS_DEF_QUERY_OK  on success or RKFS_DEF_QUERY_ERROR. on fail
 * 
 * Filesystem uses this function to send a comand (@see doc/protokol/protokol.txt).
 * When command is send, deamon sleep on this call of this function.
 * When kernel return from this function, the result of the command is in buffer and can 
 * be parsed and used.
 * 
 */
int query(const char* command, int command_length) {
  int result;
  
  FSPDEBUG("query(%s), len=%d\n", command, (int) command_length);
  result = fsc_query(command, command_length, out_buf, OUT_BUF_LENGTH);
  
  if (result > 0) {
    out_buf[result] = '\0'; /* adding trailing zero for print out, there is always enough space */
    parse_message(out_buf, result, &ret_command);
    return RKFS_DEF_QUERY_OK;
  }

  else {
   FSPDEBUG("error on query\n");
   return RKFS_DEF_QUERY_ERROR;
  }
  
}


/**
 * @brief Function for testing deamon
 * 
 * Send various messages to deamon. Function is used only for testing.
 * This time is not used at any place in the project, but we still leave it here, it can be
 * used in future when we make some changes in deamon.
 * This function is only for debugging purposis.
 * 
 */
void rkfs_deamon_test(void) {
	
/* this test will pass under condition of newly started database - no used counters!! */
#ifdef TEST_DEAMON

   query("MYSQL1.1 TS");
   query("MYSQL1.1 RE 1");
   query("MYSQL1.1 RE 8");
   query("MYSQL1.1 CF 8 hello.txt");
   query("MYSQL1.1 CF 1 hello.txt");
   query("MYSQL1.1 CF 1 hello.txt");
   query("MYSQL1.1 MD 1 novy_adresar");
   query("MYSQL1.1 MD 8 jiny_adresar");
   query("MYSQL1.1 MD 1 novy_adresar");
   query("MYSQL1.1 RF 50");
   query("MYSQL1.1 RF 2");
   query("MYSQL1.1 CF 1 vypln");
   query("MYSQL1.1 RD 1");
   query("MYSQL1.1 RD 3");
   query("MYSQL1.1 CR 49");
   query("MYSQL1.1 CR 7");
   query("MYSQL1.1 GB 1 0");
   query("MYSQL1.1 GB 1 1000");
   query("MYSQL1.1 GB 400 0");
   query("MYSQL1.1 GB 8 0");
   query("MYSQL1.1 GB 9 0");
   query("MYSQL1.1 GB 9 1");
   query("MYSQL1.1 AB 8 cd");
   query("MYSQL1.1 CR 8");
   query("MYSQL1.1 AB 8 ab");
   query("MYSQL1.1 AB 500 cd");
   query("MYSQL1.1 LU 1 xxxxx");
   query("MYSQL1.1 LU 1 todo.txt");
   query("MYSQL1.1 LU 1 novy_adresar");
   query("MYSQL1.1 LU 800 ahoj");
   query("MYSQL1.1 FI 1");
   query("MYSQL1.1 FI 2");
   query("MYSQL1.1 DI 10001");
   query("MYSQL1.1 DI 103485");
   query("MYSQL1.1 RI 4");
   query("MYSQL1.1 RI 10001");
#endif
}


/** structure for registration modul as filesystem */
struct file_system_type rkfs = {
  name:        "rkfs",
  get_sb:     rkfs_get_sb,
  kill_sb:    rkfs_kill_sb,
  owner:      THIS_MODULE
};


/**
 * @brief Read next data item from a answer received from a deamon
 * 
 * Message used for parsing data from message.
 * 
 * @param com_data_pos Position where we stop parcing last time.
 * @param ret_buf buffer for returned item
 * @return Length of read data.
 * 
 * When a answer came, it can seems like /xx/2/4/ddd/5/5/... this function is used to 
 * parce messages like this. It litle bit poiter juggling
 * 
 */
int rkfs_next_data_item(char **com_data_pos, char *ret_buf) {
  int len = 0;

  while ((**com_data_pos) != '/' && (**com_data_pos) != '\0') {
    *(ret_buf++) = *((*com_data_pos)++);
    len++;
  }
  
  *ret_buf = '\0';
  (*com_data_pos)++;
  
  return len;
}


/**
 * @brief Atoi function
 * 
 * Converts string to Integer.
 * 
 * @param str string to be converted.
 * @return Integer converted from str.
 * 
 * It has litle bit other behaviour, than usual atoi. It doesn't check any errors. 
 * When something not a number represintig char is in string, it's like it would be '0' instead.
 */
int rkfs_atoi(char* str) {
  int result = 0;
  char char_num;
  int int_num;
  
  while ((*str) != '\0') {
    char_num = *(str++);
    switch (char_num) {
      case '0': int_num = 0; break;
      case '1': int_num = 1; break;
      case '2': int_num = 2; break;
      case '3': int_num = 3; break;
      case '4': int_num = 4; break;
      case '5': int_num = 5; break;
      case '6': int_num = 6; break;
      case '7': int_num = 7; break;
      case '8': int_num = 8; break;
      case '9': int_num = 9; break;
      default:  int_num = 0; break;
    }
    result = result*10 + int_num;
  }
  return result;
}

/**
 * @brief RKFS File System initialization
 * 
 * Register RKFS file system in the system, register FS character device for communication 
 * between FS in kernel and FS Deamon in user space. Initialize the FSC device, create
 * proc entry for FSC device.
 * 
 * @return 0 on success, register_filesystem or register_chrdev error code on error.
 * 
 */
static int __init
rkfs_init_module(void) {
    int result;
    create_proc_read_entry("fsc_dev", 0, NULL, fsc_read_procmem, NULL); 
    result = register_filesystem(&rkfs);
    
    if  (result != 0) {
      printk(KERN_ERR "RKFS: unable to register RKFS filesystem.\n");
      printk(KERN_INFO "RKFS: register_filesystem error = %d.\n", result);
      return result;	
    }
    
    fsc_init(); 
    result = register_chrdev(fsc_major, CHAR_DEVICE_NAME,  &fsc_fops);
    
    if (result < 0) {
    	printk(KERN_ERR "FSC: unable to register RKFSCHAR character device. \n");
    	printk(KERN_INFO "FSC: register_chrdev error = %d.\n", result);
    	return result;
    }
     	
    if (fsc_major == 0) {
    	fsc_major = result;
    	printk(KERN_INFO "FSC: char device succesfully loaded.\n");
    }	
    
    printk(KERN_INFO "RKFS: filesystem succesfully loaded.\n");
    return 0;
}


/**
 * @brief RKFS File System finalization
 * 
 * Unregister RKFS file system from the system, unregister FS character device.
 * Removes proc entry for FSC device.
 * 
 * @return 0 on success, unregister_filesystem or unregister_chrdev error code on error.
*/
static void	 rkfs_cleanup_module(void) {
	int result;  
	remove_proc_entry("fsc_dev", NULL);
	result = unregister_filesystem(&rkfs);
	
	if (result != 0) {
		printk(KERN_ERR "RKFS: cannot be unloaded. \n");
		printk(KERN_INFO "RKFS: unregister_filesystem error = %d.\n", result);
	}
	 
	else {
		printk(KERN_INFO "RKFS: filesystem succesfully unloaded. \n");
	}
	
    fsc_fini(); 
	result = unregister_chrdev(fsc_major, CHAR_DEVICE_NAME);

	if (result != 0) {
		printk(KERN_ERR "FSC: cannot be unloaded. \n");
		printk(KERN_INFO "FSC: unregister_chrdev error = %d.\n", result);
	}
	 
	else {
		printk(KERN_INFO "FSC: char device succesfully unloaded.\n");
	}
	
}


/** Modul initialization function */
module_init(rkfs_init_module);


/** Modul exit function */
module_exit(rkfs_cleanup_module);


/**
 *
 * @}
 * 
 * @}
 * 
 */
