/**
 * @file fsc_debug.h
 * @brief Implements debugging macros for kernel part of filesystem
 * @author Michal Podzimek, Michal Ficek, Tomas Pop, MFF UK, 2007
 *
 * @addtogroup q_mysqlfs_kernel MySQL FS Kernel Module
 * @{
 * @addtogroup g_mysqlfs_kernel_degub Debug
 * @{
 * 
 */


#ifndef FSC_DEBUG_H_
 #define FSC_DEBUG_H_


#ifndef __KERNEL__
#  define __KERNEL__
#endif


#include <linux/kernel.h>


/** If defined, debugging prints will be visible. */
// #define FSC_DEBUG


/** @name Macro to help debugging. Writes messages to kmsg
 * 
 * @{
 */
#undef PDEBUG             /* undef it, just in case */
#ifdef FSC_DEBUG

	/* This one if debugging is on, and kernel space */
#	ifdef __KERNEL__
	
#		define PDEBUG(fmt, args...) printk(KERN_DEBUG "FSC: " fmt, ## args)
#		define PINFO(fmt, args...) printk(KERN_INFO "FSC: " fmt, ## args)

	/* This one for user space */
#	else
#		define PDEBUG(fmt, args...) fprintf(stderr, fmt, ## args)
#		define PINFO(fmt, args...) fprintf(stdout, fmt, ## args)
#	endif

#else
#	define PDEBUG(fmt, args...) /* not debugging: nothing */
#	define PINFO(fmt, args...) 
#endif
/** @} */

#endif


/**
 *
 * @}
 * 
 * @}
 * 
 */

