/**
 * @file fsc_device.h
 * @brief FS Char Device settings, constants and typedefs.
 * @author Michal Podzimek, Michal Ficek, Tomas Pop, MFF UK, 2007
 *
 * @addtogroup q_mysqlfs_kernel MySQL FS Kernel Module
 * @{
 * @addtogroup g_mysqlfs_kernel_chardev MySQL FS Char Device
 * @{
 *
 */


#ifndef FSC_DEVICE
 #define FSC_DEVICE


/** Macro for print query */
#define QUERY(fmt, args...) printk(KERN_DEBUG "FSC: " fmt, ## args)


/** Character device name, identifies it in /dev/ */
#define CHAR_DEVICE_NAME "fsc_dev"


/** 
 * Maximum length of each char device buffer.
 * Length defined in MYSQLFS protocol: 300 bytes for command part + 4096 bytes for data part.
 * See protocol.txt for more info.
 */
#define MAX_BUFFER_LENGTH 5192


/**
 * Char device buffer structure.
 * Used both for input and output buffer.
 */
struct buffer {
  char data[MAX_BUFFER_LENGTH];    /**< Buffer data. */
  int length;                      /**< Buffer data content lenght, NOT buffer size. */
  wait_queue_head_t deamon_queue;  /**< Wait queue for user space process, usually deamon. */
  wait_queue_head_t kernel_queue;  /**< Wait queue for kernel thread, usually char device. */
};


/* Declarations of functions provided */
int fsc_query(const char * , const int , char*, int);
int fsc_read_procmem(char *, char **, off_t , int , int *, void *);
void fsc_init(void);
void fsc_fini(void);


/** FSC Device module major number. */
extern int fsc_major;


/** FSC  Operations. */
extern struct file_operations fsc_fops;

#endif


/**
 *
 * @}
 * 
 * @}
 * 
 */
