/**
 * @file parser.h
 * @brief Header file for implementation of functions and defines and structs, file fsd_lib.c
 * @author Michal Podzimek, Michal Ficek, Tomas Pop, MFF UK, 2007
 *
 * @addtogroup q_mysqlfs_parser Parser
 * @{
 * 
 * This file contents declaration of functions and definitions of constants and structures which are
 * used in mysql fs deamon code.
 * 
 */


#ifndef PARSER_DEAMON_H_
#define PARSER_DEAMON_H_


/* parser is one for userspace and one for kernel 
 * it is ugly insert file with path, but in this case it's simpiest way. 
 */
#ifdef __KERNEL__
	#include "../kernel_module/fsc_debug.h"
#else
	#include "../deamon/fsd_debug.h"
#endif

#include "../deamon/fsd_defines.h"	


/**
 * Whem command commes from chardevice is parsed to this structure
 * 
 */
typedef struct {
	/** length of data received */
	int data_len; 				
	/** version of protokol used (MYSQLFS1.1) */			
	char version[MAX_VERSION_LENGTH + 1]; 	
	/** command, this means what should be done */
	char command[MAX_COMMAND_LENGTH + 1]; 	
	/** parametr of the command */
	char param[MAX_PARAM_LENGTH + 1];	
	/** data received  */	
	char data[MAX_BLOCK_LENGTH + 1];
	/** translation from command to a symbolic constant */	
	int command_nr; 						
} struct_command;


/* function for parsing commands */
int parse_message(char *message, int message_len, struct_command * command);


#endif


/**
 *
 * @}
 * 
 */
 
